@@ -54,10 +54,10 @@
  *	software in accordance with the terms specified in this
  *	license.
  *
- * RCS: @(#) $Id: tkMacOSXKeyEvent.c,v 1.6.2.13 2007/04/29 02:26:49 das Exp $
+ * RCS: @(#) $Id: tkMacOSXKeyEvent.c,v 1.6.2.15 2007/06/29 03:22:02 das Exp $
  */
 
-#include "tkMacOSXInt.h"
+#include "tkMacOSXPrivate.h"
 #include "tkMacOSXEvent.h"
 
 /*
@@ -178,7 +178,12 @@ TkMacOSXProcessKeyboardEvent(
 	    case kHICommandPreferences:
 	    case kHICommandQuit:
 		statusPtr->stopProcessing = 0;
-		return 0; /* TODO: may not be on event on queue. */
+
+		/*
+		 * TODO: may not be on event on queue.
+		 */
+
+		return 0;
 		break;
 	    default:
 		break;
@@ -1118,8 +1123,8 @@ TkpSetCapture(
 	    m = kWindowModalityNone;
 	}
 	if (w && w->window != None && TkMacOSXHostToplevelExists(w)) {
-	    ChkErr(SetWindowModality, GetWindowFromPort(
-		    TkMacOSXGetDrawablePort(w->window)), m, NULL);
+	    ChkErr(SetWindowModality, TkMacOSXDrawableWindow(w->window), m,
+		    NULL);
 	}
     }
 #endif
