@@ -10,7 +10,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkWinDraw.c,v 1.5 1999/08/10 05:04:56 jingham Exp $
+ * RCS: @(#) $Id: tkWinDraw.c,v 1.12.2.3 2007/12/05 19:17:32 hobbs Exp $
  */
 
 #include "tkWinInt.h"
@@ -57,7 +57,7 @@ int tkpWinRopModes[] = {
 #define SRCORREVERSE	(DWORD)0x00DD0228 /* dest = source OR (NOT dest) */
 #define SRCNAND		(DWORD)0x007700E6 /* dest = NOT (source AND dest) */
 
-static int bltModes[] = {
+int tkpWinBltModes[] = {
     BLACKNESS,			/* GXclear */
     SRCAND,			/* GXand */
     SRCERASE,			/* GXandReverse */
@@ -125,6 +125,7 @@ static void		DrawOrFillArc _ANSI_ARGS_((Display *display,
 static void		RenderObject _ANSI_ARGS_((HDC dc, GC gc,
 			    XPoint* points, int npoints, int mode, HPEN pen,
 			    WinDrawFunc func));
+static HPEN		SetUpGraphicsPort _ANSI_ARGS_((GC gc));
 
 /*
  *----------------------------------------------------------------------
@@ -172,6 +173,7 @@ TkWinGetDrawableDC(display, d, state)
 	cmap = twdPtr->bitmap.colormap;
     }
     state->palette = TkWinSelectPalette(dc, cmap);
+    state->bkmode  = GetBkMode(dc);
     return dc;
 }
 
@@ -198,6 +200,7 @@ TkWinReleaseDrawableDC(d, dc, state)
     TkWinDCState *state;
 {
     TkWinDrawable *twdPtr = (TkWinDrawable *)d;
+    SetBkMode(dc, state->bkmode);
     SelectPalette(dc, state->palette, TRUE);
     RealizePalette(dc);
     if (twdPtr->type == TWD_WINDOW) {
@@ -325,7 +328,7 @@ XCopyArea(display, src, dest, gc, src_x, src_y, width, height, dest_x, dest_y)
     }
 
     BitBlt(destDC, dest_x, dest_y, width, height, srcDC, src_x, src_y,
-	    bltModes[gc->function]);
+	    tkpWinBltModes[gc->function]);
 
     SelectClipRgn(destDC, NULL);
 
@@ -412,11 +415,14 @@ XCopyPlane(display, src, dest, gc, src_x, src_y, width, height, dest_x,
 	    /*
 	     * Case 2: transparent bitmaps are handled by setting the
 	     * destination to the foreground color whenever the source
-	     * pixel is set.
+	     * pixel is set.  We need to reset the BkColor and TextColor,
+	     * because they affect bitmap color mapping.
 	     */
 
 	    fgBrush = CreateSolidBrush(gc->foreground);
 	    oldBrush = SelectObject(destDC, fgBrush);
+	    SetBkColor(destDC, RGB(255,255,255));
+	    SetTextColor(destDC, RGB(0,0,0));
 	    BitBlt(destDC, dest_x, dest_y, width, height, srcDC, src_x, src_y,
 		    MASKPAT);
 	    SelectObject(destDC, oldBrush);
@@ -426,7 +432,7 @@ XCopyPlane(display, src, dest, gc, src_x, src_y, width, height, dest_x,
 	    /*
 	     * Case 3: two arbitrary bitmaps.  Copy the source rectangle
 	     * into a color pixmap.  Use the result as a brush when
-	     * copying the clip mask into the destination.	 
+	     * copying the clip mask into the destination.
 	     */
 
 	    HDC memDC, maskDC;
@@ -562,33 +568,7 @@ TkPutImage(colors, ncolors, display, d, gc, image, src_x, src_y, dest_x,
 	
 	infoPtr->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
 	infoPtr->bmiHeader.biWidth = image->width;
-
-	/*
-	 * The following code works around a bug in Win32s.  CreateDIBitmap
-	 * fails under Win32s for top-down images.  So we have to reverse the
-	 * order of the scanlines.  If we are not running under Win32s, we can
-	 * just declare the image to be top-down.
-	 */
-
-	if (tkpIsWin32s) {
-	    int y;
-	    char *srcPtr, *dstPtr, *temp;
-
-	    temp = ckalloc((unsigned) image->bytes_per_line);
-	    srcPtr = image->data;
-	    dstPtr = image->data+(image->bytes_per_line * (image->height - 1));
-	    for (y = 0; y < (image->height/2); y++) {
-		memcpy(temp, srcPtr, image->bytes_per_line);
-		memcpy(srcPtr, dstPtr, image->bytes_per_line);
-		memcpy(dstPtr, temp, image->bytes_per_line);
-		srcPtr += image->bytes_per_line;
-		dstPtr -= image->bytes_per_line;
-	    }
-	    ckfree(temp);
-	    infoPtr->bmiHeader.biHeight = image->height; /* Bottom-up order */
-	} else {
-	    infoPtr->bmiHeader.biHeight = -image->height; /* Top-down order */
-	}
+	infoPtr->bmiHeader.biHeight = -image->height; /* Top-down order */
 	infoPtr->bmiHeader.biPlanes = 1;
 	infoPtr->bmiHeader.biBitCount = image->bits_per_pixel;
 	infoPtr->bmiHeader.biCompression = BI_RGB;
@@ -612,6 +592,12 @@ TkPutImage(colors, ncolors, display, d, gc, image, src_x, src_y, dest_x,
 		image->data, infoPtr, DIB_RGB_COLORS);
 	ckfree((char *) infoPtr);
     }
+    if(!bitmap) {
+	panic("Fail to allocate bitmap\n");
+	DeleteDC(dcMem);
+    	TkWinReleaseDrawableDC(d, dc, &state);
+	return;
+    }
     bitmap = SelectObject(dcMem, bitmap);
     BitBlt(dc, dest_x, dest_y, width, height, dcMem, src_x, src_y, SRCCOPY);
     DeleteObject(SelectObject(dcMem, bitmap));
@@ -647,7 +633,7 @@ XFillRectangles(display, d, gc, rectangles, nrectangles)
     int i;
     RECT rect;
     TkWinDCState state;
-    HBRUSH brush;
+    HBRUSH brush, oldBrush;
 
     if (d == None) {
 	return;
@@ -661,7 +647,7 @@ XFillRectangles(display, d, gc, rectangles, nrectangles)
 	    || gc->fill_style == FillOpaqueStippled)
 	    && gc->stipple != None) {
 	TkWinDrawable *twdPtr = (TkWinDrawable *)gc->stipple;
-	HBRUSH oldBrush, stipple;
+	HBRUSH stipple;
 	HBITMAP oldBitmap, bitmap;
 	HDC dcMem;
 	HBRUSH bgBrush = CreateSolidBrush(gc->background);
@@ -711,9 +697,28 @@ XFillRectangles(display, d, gc, rectangles, nrectangles)
 	DeleteObject(stipple);
 	DeleteObject(bgBrush);
     } else {
-	for (i = 0; i < nrectangles; i++) {
-	    TkWinFillRect(dc, rectangles[i].x, rectangles[i].y,
-		    rectangles[i].width, rectangles[i].height, gc->foreground);
+	if (gc->function == GXcopy) {
+	    for (i = 0; i < nrectangles; i++) {
+		rect.left = rectangles[i].x;
+		rect.right = rect.left + rectangles[i].width;
+		rect.top = rectangles[i].y;
+		rect.bottom = rect.top + rectangles[i].height;
+		FillRect(dc, &rect, brush);
+	    }
+	} else {
+	    HPEN newPen = CreatePen(PS_NULL, 0, gc->foreground);
+	    HPEN oldPen = SelectObject(dc, newPen);
+	    oldBrush = SelectObject(dc, brush);
+	    
+	    for (i = 0; i < nrectangles; i++) {
+		Rectangle(dc, rectangles[i].x, rectangles[i].y,
+		    rectangles[i].x + rectangles[i].width + 1,
+		    rectangles[i].y + rectangles[i].height + 1);
+	    }
+
+	    SelectObject(dc, oldBrush);
+	    SelectObject(dc, oldPen);
+	    DeleteObject(newPen);
 	}
     }
     DeleteObject(brush);
@@ -768,15 +773,13 @@ RenderObject(dc, gc, points, npoints, mode, pen, func)
 	}
 
 	/*
-	 * Grow the bounding box enough to account for wide lines.
+	 * Grow the bounding box enough to account for line width.
 	 */
 
-	if (gc->line_width > 1) {
-	    rect.left -= gc->line_width;
-	    rect.top -= gc->line_width;
-	    rect.right += gc->line_width;
-	    rect.bottom += gc->line_width;
-	}
+	rect.left -= gc->line_width;
+	rect.top -= gc->line_width;
+	rect.right += gc->line_width;
+	rect.bottom += gc->line_width;
 
 	width = rect.right - rect.left;
 	height = rect.bottom - rect.top;
@@ -888,42 +891,8 @@ XDrawLines(display, d, gc, points, npoints, mode)
 
     dc = TkWinGetDrawableDC(display, d, &state);
 
-    if (!tkpIsWin32s && (gc->line_width > 1)) {
-	LOGBRUSH lb;
-	DWORD style;
-
-	lb.lbStyle = BS_SOLID;
-	lb.lbColor = gc->foreground;
-	lb.lbHatch = 0;
-
-	style = PS_GEOMETRIC|PS_COSMETIC;
-	switch (gc->cap_style) {
-	    case CapNotLast:
-	    case CapButt:
-		style |= PS_ENDCAP_FLAT; 
-		break;
-	    case CapRound:
-		style |= PS_ENDCAP_ROUND; 
-		break;
-	    default:
-		style |= PS_ENDCAP_SQUARE; 
-		break;
-	}
-	switch (gc->join_style) {
-	    case JoinMiter: 
-		style |= PS_JOIN_MITER; 
-		break;
-	    case JoinRound:
-		style |= PS_JOIN_ROUND; 
-		break;
-	    default:
-		style |= PS_JOIN_BEVEL; 
-		break;
-	}
-	pen = ExtCreatePen(style, gc->line_width, &lb, 0, NULL);
-    } else {
-	pen = CreatePen(PS_SOLID, gc->line_width, gc->foreground);
-    }
+    pen = SetUpGraphicsPort(gc);
+    SetBkMode(dc, TRANSPARENT);
     RenderObject(dc, gc, points, npoints, mode, pen, Polyline);
     DeleteObject(pen);
     
@@ -1009,7 +978,8 @@ XDrawRectangle(display, d, gc, x, y, width, height)
 
     dc = TkWinGetDrawableDC(display, d, &state);
 
-    pen = CreatePen(PS_SOLID, gc->line_width, gc->foreground);
+    pen = SetUpGraphicsPort(gc);
+    SetBkMode(dc, TRANSPARENT);
     oldPen = SelectObject(dc, pen);
     oldBrush = SelectObject(dc, GetStockObject(NULL_BRUSH));
     SetROP2(dc, tkpWinRopModes[gc->function]);
@@ -1171,7 +1141,7 @@ DrawOrFillArc(display, d, gc, x, y, width, height, start, extent, fill)
      * difference in pixel definitions between X and Windows.
      */
 
-    pen = CreatePen(PS_SOLID, gc->line_width, gc->foreground);
+    pen = SetUpGraphicsPort(gc);
     oldPen = SelectObject(dc, pen);
     if (!fill) {
 	/*
@@ -1180,6 +1150,7 @@ DrawOrFillArc(display, d, gc, x, y, width, height, start, extent, fill)
 	 * it's only supported under Windows NT.
 	 */
 
+	SetBkMode(dc, TRANSPARENT);
 	Arc(dc, x, y, x+width+1, y+height+1, xstart, ystart, xend, yend);
     } else {
 	brush = CreateSolidBrush(gc->foreground);
@@ -1195,6 +1166,92 @@ DrawOrFillArc(display, d, gc, x, y, width, height, start, extent, fill)
     TkWinReleaseDrawableDC(d, dc, &state);
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * SetUpGraphicsPort --
+ *
+ *	Set up the graphics port from the given GC.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	The current port is adjusted.
+ *
+ *----------------------------------------------------------------------
+ */
+
+static HPEN
+SetUpGraphicsPort(gc)
+    GC gc;
+{
+    DWORD style;
+
+    if (gc->line_style == LineOnOffDash) {
+	unsigned char *p = (unsigned char *) &(gc->dashes);
+				/* pointer to the dash-list */
+
+	/*
+	 * Below is a simple translation of serveral dash patterns
+	 * to valid windows pen types. Far from complete,
+	 * but I don't know how to do it better.
+	 * Any ideas: <mailto:j.nijtmans@chello.nl>
+	 */
+
+	if (p[1] && p[2]) {
+	    if (!p[3] || p[4]) {
+		style = PS_DASHDOTDOT;		/*	-..	*/
+	    } else {
+		style = PS_DASHDOT;		/*	-.	*/
+	    }
+	} else {
+	    if (p[0] > (4 * gc->line_width)) {
+		style = PS_DASH;		/*	-	*/
+	    } else {
+		style = PS_DOT;			/*	.	*/
+	    }
+	}
+    } else {
+	style = PS_SOLID;
+    }
+    if (gc->line_width < 2) {
+	return CreatePen(style, gc->line_width, gc->foreground);
+    } else {
+	LOGBRUSH lb;
+
+	lb.lbStyle = BS_SOLID;
+	lb.lbColor = gc->foreground;
+	lb.lbHatch = 0;
+
+	style |= PS_GEOMETRIC;
+	switch (gc->cap_style) {
+	    case CapNotLast:
+	    case CapButt:
+		style |= PS_ENDCAP_FLAT; 
+		break;
+	    case CapRound:
+		style |= PS_ENDCAP_ROUND; 
+		break;
+	    default:
+		style |= PS_ENDCAP_SQUARE; 
+		break;
+	}
+	switch (gc->join_style) {
+	    case JoinMiter: 
+		style |= PS_JOIN_MITER; 
+		break;
+	    case JoinRound:
+		style |= PS_JOIN_ROUND; 
+		break;
+	    default:
+		style |= PS_JOIN_BEVEL; 
+		break;
+	}
+	return ExtCreatePen(style, gc->line_width, &lb, 0, NULL);
+    }
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1269,7 +1326,7 @@ TkWinFillRect(dc, x, y, width, height, pixel)
     ExtTextOut(dc, 0, 0, ETO_OPAQUE, &rect, NULL, 0, NULL);
     SetBkColor(dc, oldColor);
 }
-
+
 /*
  *----------------------------------------------------------------------
  *
@@ -1293,13 +1350,39 @@ TkWinFillRect(dc, x, y, width, height, pixel)
  */
 
 void 
-TkpDrawHighlightBorder (
-        Tk_Window tkwin, 
-        GC fgGC, 
-        GC bgGC, 
-        int highlightWidth,
-        Drawable drawable)
+TkpDrawHighlightBorder(tkwin, fgGC, bgGC, highlightWidth, drawable)
+    Tk_Window tkwin;
+    GC fgGC;
+    GC bgGC;
+    int highlightWidth;
+    Drawable drawable;
 {
-    TkDrawInsetFocusHighlight (tkwin, fgGC, highlightWidth, drawable, 0);
+    TkDrawInsetFocusHighlight(tkwin, fgGC, highlightWidth, drawable, 0);
+}
+
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkpDrawFrame --
+ *
+ *	This procedure draws the rectangular frame area.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Draws inside the tkwin area.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+TkpDrawFrame (Tk_Window tkwin, Tk_3DBorder border,
+	int highlightWidth, int borderWidth, int relief)
+{
+    Tk_Fill3DRectangle(tkwin, Tk_WindowId(tkwin),
+	    border, highlightWidth, highlightWidth,
+	    Tk_Width(tkwin) - 2 * highlightWidth,
+	    Tk_Height(tkwin) - 2 * highlightWidth,
+	    borderWidth, relief);
 }
-        
\ No newline at end of file
