@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkWinKey.c,v 1.12.2.1 2001/04/04 07:57:18 hobbs Exp $
+ * RCS: @(#) $Id: tkWinKey.c,v 1.14.4.1 2006/08/30 21:53:46 hobbs Exp $
  */
 
 #include "tkWinInt.h"
@@ -112,6 +112,13 @@ TkpGetString(winPtr, eventPtr, dsPtr)
         len = Tcl_UniCharToUtf((Tcl_UniChar) unichar, buf);
 
         Tcl_DStringAppend(dsPtr, buf, len);
+    } else if (eventPtr->xkey.send_event == -3) {
+	/*
+	 * Special case for WM_UNICHAR.
+	 * xkey.trans_chars[] already contains a UTF-8 char.
+	 */
+	Tcl_DStringAppend(dsPtr, eventPtr->xkey.trans_chars,
+		eventPtr->xkey.nbytes);
     } else  {
 	/*
 	 * This is an event generated from generic code.  It has no
@@ -584,14 +591,6 @@ TkpSetKeycodeAndState(tkwin, keySym, eventPtr)
             eventPtr->xkey.keycode = (KeyCode) (result & 0xff);
 	}
     }
-    {
-        /* Debug log */
-        FILE *fp = fopen("c:\\temp\\tklog.txt", "a");
-        if (fp != NULL) {
-            fprintf(fp, "TkpSetKeycode. Keycode %d State %d Keysym %d\n", eventPtr->xkey.keycode, eventPtr->xkey.state, keySym);
-            fclose(fp);
-        }
-    }
 }
 
 /*
